/*
 *  App.java
 *
 *  Copyright (C) 2010  Sérgio Lopes
 *
 *  This file is part of AgendaDB4O.
 *
 *  AgendaDB4O is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  AgendaDB4O is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with AgendaDB4O. If not, see <http://www.gnu.org/licenses/gpl.html>.
 */
package org.sergiolopes.agendabd4o;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.UIManager;

/**
 * Interface gráfica principal.
 * 
 * @author Sérgio Lopes
 */
public class App extends javax.swing.JFrame {

    private App me = this;
    private Agenda agenda;
    private ModeloTabelaContacto modelo;
    //Variáveis de apoio
    private Contacto seleccionado;

    /** Creates new form App */
    public App() {
        modelo = new ModeloTabelaContacto(null);
        initComponents();

        //NOTA: este código deveria estar dentro do método iniComponents
        jtTabelaContactos.setBorder(null);
        jspScroll.setBorder(null);
    }

    /**
     * Efectua uma pesquisa.
     */
    private void pesquisar() {
        modelo.setPesquisa(jtfPesquisa.getText().trim());
    }

    /**
     * Permite gravar dados quando os mesmos são alterados na janela de diálogo
     * <em>Cartao</em> ou quando é criado um novo contcto.
     *
     * @param contacto Contacto a gravar ou actualizar.
     * @param novo Indica se o contacto é novo.
     */
    public void gravarDados(Contacto contacto, boolean novo) {
        if (novo) {
            agenda.adicionarContacto(contacto);
        } else {
            agenda.actualizarContacto(contacto);
        }
    }

    /**
     * Altera o estado de disponibilidade dos botões.
     *
     * @param estado Se true activa os botões, caso contrário desactiva-os.
     */
    private void estadoUI(boolean estado) {
        jbtnAdicionarContacto.setEnabled(estado);
        jbtnPesquisar.setEnabled(estado);
        jbtnRemoverContacto.setEnabled(estado);
        jtfPesquisa.setEditable(estado);
    }

    /**
     * Modificação para permitir centrar a janela.
     *
     * @param visible Se verdadeiro torna a janela visível, esconde a janela em
     * caso contrário
     * @see Window.setVisible
     */
    @Override
    public void setVisible(boolean visible) {
        Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension mySize = getSize();

        if (mySize.height > scrSize.height) {
            mySize.height = scrSize.height;
        }

        if (mySize.width > scrSize.width) {
            mySize.width = scrSize.width;
        }

        setLocation((scrSize.width - mySize.width) / 2,
                (scrSize.height - mySize.height) / 2);


        super.setVisible(visible);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jtbToolbar = new javax.swing.JToolBar();
        jbtnNovaAgenda = new javax.swing.JButton();
        jbtnAbrirAgenda = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        jbtnAdicionarContacto = new javax.swing.JButton();
        jbtnRemoverContacto = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        jtfPesquisa = new javax.swing.JTextField();
        jbtnPesquisar = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        jbtnSobre = new javax.swing.JButton();
        jbtnSair = new javax.swing.JButton();
        jspScroll = new javax.swing.JScrollPane();
        jtTabelaContactos = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Agenda DB4O");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jtbToolbar.setFloatable(false);
        jtbToolbar.setRollover(true);

        jbtnNovaAgenda.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/easymoblog.png"))); // NOI18N
        jbtnNovaAgenda.setToolTipText("Nova agenda ...");
        jbtnNovaAgenda.setFocusable(false);
        jbtnNovaAgenda.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnNovaAgenda.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnNovaAgenda.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnNovaAgendaActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnNovaAgenda);

        jbtnAbrirAgenda.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/contents.png"))); // NOI18N
        jbtnAbrirAgenda.setToolTipText("Abrir agenda ...");
        jbtnAbrirAgenda.setFocusable(false);
        jbtnAbrirAgenda.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAbrirAgenda.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnAbrirAgenda.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAbrirAgendaActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnAbrirAgenda);
        jtbToolbar.add(jSeparator1);

        jbtnAdicionarContacto.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/add_user.png"))); // NOI18N
        jbtnAdicionarContacto.setToolTipText("Adicionar contacto ...");
        jbtnAdicionarContacto.setEnabled(false);
        jbtnAdicionarContacto.setFocusable(false);
        jbtnAdicionarContacto.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAdicionarContacto.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnAdicionarContacto.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarContactoActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnAdicionarContacto);

        jbtnRemoverContacto.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/delete_user.png"))); // NOI18N
        jbtnRemoverContacto.setToolTipText("Remover contacto");
        jbtnRemoverContacto.setEnabled(false);
        jbtnRemoverContacto.setFocusable(false);
        jbtnRemoverContacto.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnRemoverContacto.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnRemoverContacto.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnRemoverContactoActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnRemoverContacto);
        jtbToolbar.add(jSeparator2);

        jtfPesquisa.setEditable(false);
        jtfPesquisa.setForeground(new java.awt.Color(255, 255, 255));
        jtfPesquisa.setCursor(new java.awt.Cursor(java.awt.Cursor.TEXT_CURSOR));
        jtfPesquisa.setDragEnabled(false);
        jtfPesquisa.setMaximumSize(new java.awt.Dimension(200, 28));
        jtfPesquisa.setPreferredSize(new java.awt.Dimension(50, 36));
        jtfPesquisa.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jtfPesquisaActionPerformed(evt);
            }
        });
        jtbToolbar.add(jtfPesquisa);

        jbtnPesquisar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/search.png"))); // NOI18N
        jbtnPesquisar.setToolTipText("Pesquisar");
        jbtnPesquisar.setEnabled(false);
        jbtnPesquisar.setFocusable(false);
        jbtnPesquisar.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnPesquisar.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnPesquisar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnPesquisarActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnPesquisar);
        jtbToolbar.add(jSeparator4);

        jbtnSobre.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/info.png"))); // NOI18N
        jbtnSobre.setToolTipText("Sobre");
        jbtnSobre.setFocusable(false);
        jbtnSobre.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnSobre.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnSobre.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnSobreActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnSobre);

        jbtnSair.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/exit.png"))); // NOI18N
        jbtnSair.setToolTipText("Sair");
        jbtnSair.setFocusable(false);
        jbtnSair.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnSair.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnSair.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnSairActionPerformed(evt);
            }
        });
        jtbToolbar.add(jbtnSair);

        jspScroll.setOpaque(false);

        jtTabelaContactos.setModel(modelo);
        jtTabelaContactos.setOpaque(false);
        jtTabelaContactos.setShowVerticalLines(false);
        jtTabelaContactos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jtTabelaContactosMouseClicked(evt);
            }
        });
        jspScroll.setViewportView(jtTabelaContactos);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(20, 20, 20)
                .add(jspScroll, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 547, Short.MAX_VALUE)
                .add(20, 20, 20))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jtbToolbar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 587, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(jtbToolbar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jspScroll, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 405, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jbtnSairActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnSairActionPerformed
        if (agenda != null) {
            agenda.fechar();
        }

        dispose();
        System.exit(0);
    }//GEN-LAST:event_jbtnSairActionPerformed

    private void jbtnNovaAgendaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnNovaAgendaActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setFileFilter(new AgendaFileFilter());
        jfc.setFileView(new AgendaFileView());
        jfc.setMultiSelectionEnabled(false);

        if (jfc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            //Martelar a adição da extensão
            String path = jfc.getSelectedFile().getAbsolutePath();
            if (!path.endsWith(".dbo")) {
                path = path + ".dbo";
            }

            agenda = new Agenda(path);
            modelo.setAgenda(agenda);
            agenda.adicionarListener(modelo);

            estadoUI(true);
        }
    }//GEN-LAST:event_jbtnNovaAgendaActionPerformed

    private void jbtnAbrirAgendaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAbrirAgendaActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setFileFilter(new AgendaFileFilter());
        jfc.setFileView(new AgendaFileView());
        jfc.setMultiSelectionEnabled(false);

        if (jfc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            if (agenda != null) {
                agenda.fechar();
            }

            agenda = new Agenda(jfc.getSelectedFile().getAbsolutePath());
            modelo.setAgenda(agenda);
            agenda.adicionarListener(modelo);

            estadoUI(true);
        }
    }//GEN-LAST:event_jbtnAbrirAgendaActionPerformed

    private void jbtnAdicionarContactoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarContactoActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new Cartao(me, true, null).setVisible(true);
            }
        });
    }//GEN-LAST:event_jbtnAdicionarContactoActionPerformed

    private void jbtnRemoverContactoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnRemoverContactoActionPerformed
        int index = jtTabelaContactos.getSelectedRow();

        if (index >= 0) {
            agenda.removerContacto(modelo.getContactoEm(index));
        }
    }//GEN-LAST:event_jbtnRemoverContactoActionPerformed

    private void jtfPesquisaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jtfPesquisaActionPerformed
        pesquisar();
    }//GEN-LAST:event_jtfPesquisaActionPerformed

    private void jbtnSobreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnSobreActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new Sobre(me, true).setVisible(true);
            }
        });
    }//GEN-LAST:event_jbtnSobreActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        if (agenda != null) {
            agenda.fechar();
        }
    }//GEN-LAST:event_formWindowClosing

    private void jbtnPesquisarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnPesquisarActionPerformed
        pesquisar();
    }//GEN-LAST:event_jbtnPesquisarActionPerformed

    private void jtTabelaContactosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jtTabelaContactosMouseClicked
        //if (evt.getClickCount() == 2) {

            seleccionado = null;
            int index = jtTabelaContactos.getSelectedRow();

            if (index >= 0) {
                seleccionado = modelo.getContactoEm(index);
            }

            if (seleccionado != null) {
                java.awt.EventQueue.invokeLater(new Runnable() {

                    public void run() {
                        new Cartao(me, true, seleccionado).setVisible(true);
                    }
                });
            }
        //}
    }//GEN-LAST:event_jtTabelaContactosMouseClicked

    /**
     * Inicia a aplicação.
     *
     * @param args argumentos passados na linha de comandos.
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                try {
                    UIManager.setLookAndFeel("org.jvnet.substance.api.skin.SubstanceGeminiLookAndFeel");

                    //NOTA: O tratamento de excepções não está correcto. Foi tomado o caminho mais curto e não o ideal.
                } catch (Exception ex) {
                    Logger.getLogger(App.class.getName()).log(Level.WARNING, "Não foi possível definir o L&F da applicação.", ex);
                    try {
                        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    } catch (Exception ex2) {
                        Logger.getLogger(App.class.getName()).log(Level.WARNING, "Não foi possível definir o L&F nativo.", ex2);
                        try {
                            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                        } catch (Exception ex3) {
                            //NO L&F?! Where are we running this?
                            Logger.getLogger(App.class.getName()).log(Level.WARNING, "Nenhum L&F foi definido para esta aplicação.", ex3);
                        }
                    }
                }

                new App().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JButton jbtnAbrirAgenda;
    private javax.swing.JButton jbtnAdicionarContacto;
    private javax.swing.JButton jbtnNovaAgenda;
    private javax.swing.JButton jbtnPesquisar;
    private javax.swing.JButton jbtnRemoverContacto;
    private javax.swing.JButton jbtnSair;
    private javax.swing.JButton jbtnSobre;
    private javax.swing.JScrollPane jspScroll;
    private javax.swing.JTable jtTabelaContactos;
    private javax.swing.JToolBar jtbToolbar;
    private javax.swing.JTextField jtfPesquisa;
    // End of variables declaration//GEN-END:variables
}
