/*
 *  Cartao.java
 *
 *  Copyright (C) 2010  Sérgio Lopes
 *
 *  This file is part of AgendaDB4O.
 *
 *  AgendaDB4O is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  AgendaDB4O is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with AgendaDB4O. If not, see <http://www.gnu.org/licenses/gpl.html>.
 */
package org.sergiolopes.agendabd4o;

import java.awt.event.KeyEvent;

/**
 * Representa uma janela que permite editar os dados de um contacto, possibilitando
 * a criação de novos contactos ou a edição de contactos existentes.
 *
 * @author Sérgio Lopes
 */
public class Cartao extends javax.swing.JDialog {

    private Contacto contacto;
    private String titulo;

    /** Creates new form Cartao */
    public Cartao(java.awt.Frame parent, boolean modal, Contacto contacto) {
        super(parent, modal);
        this.contacto = contacto;

        if (contacto != null) {
            titulo = contacto.getNome();
        } else {
            titulo = "Novo Contacto";
        }
        setTitle(titulo);

        initComponents();
        if (contacto != null) {
            jffCodigoPostal.setText(contacto.getCodigoPostal());
            jffDataNascimento.setText(contacto.getDataNascimento());
            jtaEndereco.setText(contacto.getEndereco());
            jtfApelido.setText(contacto.getApelido());
            jtfLocalidade.setText(contacto.getLocalidade());
            jtfNome.setText(contacto.getNome());
            jtfTelefone.setText(contacto.getTelefone());
            jtfTelemovel.setText(contacto.getTelemovel());
        }

        validate();
        pack();
    }

    @Override
    public void setVisible(boolean visible) {
        setLocation(getParent().getX() + (getParent().getWidth() / 2) - (getWidth() / 2),
                getParent().getY() + (getParent().getHeight() / 2) - (getHeight() / 2));

        super.setVisible(true);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpPainelPrincipal = new javax.swing.JPanel();
        lblNome = new javax.swing.JLabel();
        jtfNome = new javax.swing.JTextField();
        lblApelido = new javax.swing.JLabel();
        jtfApelido = new javax.swing.JTextField();
        jlblDataNascimento = new javax.swing.JLabel();
        jtfTelefone = new javax.swing.JTextField();
        jlblTelefone = new javax.swing.JLabel();
        jtfTelemovel = new javax.swing.JTextField();
        jlblTelemovel = new javax.swing.JLabel();
        jspScollEndereco = new javax.swing.JScrollPane();
        jtaEndereco = new javax.swing.JTextArea();
        jtfLocalidade = new javax.swing.JTextField();
        jlblEndereco = new javax.swing.JLabel();
        jlblCodigoPostal = new javax.swing.JLabel();
        jlblLocalidade = new javax.swing.JLabel();
        jffDataNascimento = new javax.swing.JFormattedTextField();
        jffCodigoPostal = new javax.swing.JFormattedTextField();
        jbtnFecharCartao = new javax.swing.JButton();
        jbtnGravarDados = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
        });

        jpPainelPrincipal.setBorder(javax.swing.BorderFactory.createTitledBorder(titulo));

        lblNome.setText("Nome:");

        lblApelido.setText("Apelido:");

        jlblDataNascimento.setText("Data Nasc.:");

        jlblTelefone.setText("Telefone:");

        jlblTelemovel.setText("Telemóvel:");

        jtaEndereco.setColumns(20);
        jtaEndereco.setRows(5);
        jspScollEndereco.setViewportView(jtaEndereco);

        jlblEndereco.setText("Endereço:");

        jlblCodigoPostal.setText("Cód. Postal:");

        jlblLocalidade.setText("Localidade:");

        try {
            jffDataNascimento.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.MaskFormatter("##/##/####")));
        } catch (java.text.ParseException ex) {
            ex.printStackTrace();
        }

        try {
            jffCodigoPostal.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.MaskFormatter("####-###")));
        } catch (java.text.ParseException ex) {
            ex.printStackTrace();
        }

        org.jdesktop.layout.GroupLayout jpPainelPrincipalLayout = new org.jdesktop.layout.GroupLayout(jpPainelPrincipal);
        jpPainelPrincipal.setLayout(jpPainelPrincipalLayout);
        jpPainelPrincipalLayout.setHorizontalGroup(
            jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jpPainelPrincipalLayout.createSequentialGroup()
                .addContainerGap()
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(lblNome)
                    .add(lblApelido)
                    .add(jlblDataNascimento)
                    .add(jlblTelefone)
                    .add(jlblTelemovel)
                    .add(jlblEndereco)
                    .add(jlblCodigoPostal))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jpPainelPrincipalLayout.createSequentialGroup()
                        .add(jffCodigoPostal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 88, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jlblLocalidade)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jtfLocalidade, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE))
                    .add(jtfNome, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 292, Short.MAX_VALUE)
                    .add(jtfApelido, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 292, Short.MAX_VALUE)
                    .add(jspScollEndereco, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 292, Short.MAX_VALUE)
                    .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jffDataNascimento, 0, 0, Short.MAX_VALUE)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jtfTelefone)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jtfTelemovel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 119, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jpPainelPrincipalLayout.setVerticalGroup(
            jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jpPainelPrincipalLayout.createSequentialGroup()
                .addContainerGap()
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblNome)
                    .add(jtfNome, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblApelido)
                    .add(jtfApelido, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jlblDataNascimento)
                    .add(jffDataNascimento, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jtfTelefone, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jlblTelefone))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jtfTelemovel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jlblTelemovel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jpPainelPrincipalLayout.createSequentialGroup()
                        .add(jspScollEndereco, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jpPainelPrincipalLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jtfLocalidade, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jlblCodigoPostal)
                            .add(jffCodigoPostal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jlblLocalidade)))
                    .add(jlblEndereco))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jbtnFecharCartao.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/cancel.png"))); // NOI18N
        jbtnFecharCartao.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnFecharCartaoActionPerformed(evt);
            }
        });

        jbtnGravarDados.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/sergiolopes/agendabd4o/images/large/ok.png"))); // NOI18N
        jbtnGravarDados.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnGravarDadosActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jpPainelPrincipal, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(jbtnGravarDados)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jbtnFecharCartao)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(20, 20, 20)
                .add(jpPainelPrincipal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jbtnFecharCartao)
                    .add(jbtnGravarDados))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jbtnFecharCartaoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnFecharCartaoActionPerformed
        dispose();
    }//GEN-LAST:event_jbtnFecharCartaoActionPerformed

    /**
     * Grava os dados do contacto. No fim fecha a janela.
     * 
     * @param evt Este parâmetro é ignorado.
     */
    private void jbtnGravarDadosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnGravarDadosActionPerformed
        boolean novo = false;
        String nome = jtfNome.getText().trim();
        String apelido = jtfApelido.getText().trim();
        String dataNascimento = jffDataNascimento.getText().trim();
        String telefone = jtfTelefone.getText().trim();
        String telemovel = jtfTelemovel.getText().trim();
        String endereco = jtaEndereco.getText().trim();
        String codigoPostal = jffCodigoPostal.getText().trim();
        String localidade = jtfLocalidade.getText().trim();

        if (contacto == null) {
            novo = true;
            contacto = new Contacto(nome, apelido, dataNascimento, telefone,
                    telemovel, endereco,
                    codigoPostal, localidade);
        } else {
            contacto.setNome(nome);
            contacto.setApelido(apelido);
            contacto.setDataNascimento(dataNascimento);
            contacto.setTelefone(telefone);
            contacto.setTelemovel(telemovel);
            contacto.setEndereco(endereco);
            contacto.setCodigoPostal(codigoPostal);
            contacto.setLocalidade(localidade);
        }

        ((App) getParent()).gravarDados(contacto, novo);
        dispose();
    }//GEN-LAST:event_jbtnGravarDadosActionPerformed

    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
        if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
            dispose();
        }
    }//GEN-LAST:event_formKeyPressed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jbtnFecharCartao;
    private javax.swing.JButton jbtnGravarDados;
    private javax.swing.JFormattedTextField jffCodigoPostal;
    private javax.swing.JFormattedTextField jffDataNascimento;
    private javax.swing.JLabel jlblCodigoPostal;
    private javax.swing.JLabel jlblDataNascimento;
    private javax.swing.JLabel jlblEndereco;
    private javax.swing.JLabel jlblLocalidade;
    private javax.swing.JLabel jlblTelefone;
    private javax.swing.JLabel jlblTelemovel;
    private javax.swing.JPanel jpPainelPrincipal;
    private javax.swing.JScrollPane jspScollEndereco;
    private javax.swing.JTextArea jtaEndereco;
    private javax.swing.JTextField jtfApelido;
    private javax.swing.JTextField jtfLocalidade;
    private javax.swing.JTextField jtfNome;
    private javax.swing.JTextField jtfTelefone;
    private javax.swing.JTextField jtfTelemovel;
    private javax.swing.JLabel lblApelido;
    private javax.swing.JLabel lblNome;
    // End of variables declaration//GEN-END:variables
}
